package org.eclipse.spliteditor.tests;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Sash;


public class SplittableContainer extends Composite {

	SplittableContainerClient client;

	Sash sash;
	
	int splitRatio;
	int splitTotal;
	
	Control master;
	Control slave;
	
	int sashWidth;
	
	int dragMinimum;

	public SplittableContainer(Composite parent, SplittableContainerClient client, int style) {
		super(parent, style);
		super.setLayout(new SplitLayout());

		this.sashWidth = 3;

		this.dragMinimum = 10;

		this.client = client;
		
		master = client.createMaster(this);

		sash = new Sash(this, SWT.HORIZONTAL);
		sash.setSize(5, 5);
		sash.addListener (SWT.Selection, new SashSelectionListener());
	}

	public void setLayout(Layout layout) {
		checkWidget();
	}

	public int getSashWidth() {
		return sashWidth;
	}

	public void setSashWidth(int sashWidth) {
		if (sashWidth == this.sashWidth) return;
		this.sashWidth = sashWidth;
		layout(false);
	}

	public int getDragMinimum() {
		return dragMinimum;
	}

	public void setDragMinimum(int dragMinimum) {
		this.dragMinimum = dragMinimum;
	}

	public Control getMaster() {
		return master;
	}

	public Control getSlave() {
		return slave;
	}

	private final class SashSelectionListener implements Listener {
		public void handleEvent (Event e) {
			if ((e.detail & SWT.DRAG) == 0) {
				splitRatio = e.y;
				splitTotal = SplittableContainer.this.getSize().y - sashWidth;
				
				if (splitRatio < dragMinimum) {
					splitRatio = 0;
					if (slave != null) {
						slave.dispose();
						slave = null;
					}
				} else {
					if (slave == null) {
						slave = client.createSlave(SplittableContainer.this);
					}
				}
				
				SplittableContainer.this.layout();
			}
		}
	}

	private final class SplitLayout extends Layout {

		protected Point computeSize(Composite composite, int hint, int hint2,
				boolean flushCache) {
			return null;
		}

		protected void layout(Composite composite, boolean flushCache) {
			Rectangle area = composite.getClientArea();
			if (area.width <= 1 || area.height <= 1) return;
			
			// Calculate split position
			if (splitTotal == 0) {
				splitTotal = area.height;
			}
			int sashPos = (splitRatio * (area.height - sashWidth)) / splitTotal;
			
			// Layout slave
			if (slave != null) {
				slave.setBounds(area.x, area.y, area.width, sashPos);
			}
			
			// Layout sash
			sash.setBounds(area.x, area.y + sashPos, area.width, sashWidth);
			int sashBottom = sashPos + sashWidth;
			
			// Layout master
			master.setBounds(area.x, area.y + sashBottom, area.width, area.height - sashBottom);
		}

	}

}
